Schema = [
    {"header_msg": {
        "header": {
            "reference": None,
            "created": None,
            "user_handle": None,
            "app_handle": None,
            "version": "0.2",
            "crypto": "ETH"
        },
        "message": "header_msg",
        "kyc_level": None,
        "entity_type": None
    }},
    {"link_account_msg_plaid": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "version": "0.2",
            "crypto": "ETH",
            "reference": None
        },
        "message": "link_account_msg",
        "plaid_token": None,
        "account_name": None,
        "selected_account_id": None,
        "plaid_token_type": None
    }},
    {"link_account_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "version": "0.2",
            "crypto": "ETH",
            "reference": None
        },
        "message": "link_account_msg",
        "account_number": None,
        "routing_number": None,
        "account_type": None,
        "account_name": None,
        "selected_account_id": None
    }},
    {"account_name_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "reference": None
        },
        "account_name": None
    }},
    {"transfer_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "version": "0.2",
            "crypto": "ETH",
            "reference": None
        },
        "message": "transfer_msg",
        "amount": None,
        "destination": None,
        "destination_handle": None,
        "destination_wallet": None,
        "destination_address": None,
        "business_uuid": None,
        "descriptor": None,
        "source_id": None,
        "destination_id": None,
    }},
    {"issue_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "version": "0.2",
            "crypto": "ETH",
            "reference": None
        },
        "message": "issue_msg",
        "amount": None,
        "account_name": None,
        "card_name": None,
        "business_uuid": None,
        "descriptor": None,
        "processing_type": None,
        "source_id": None,
        "destination_id": None,
    }},
    {"redeem_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "version": "0.2",
            "crypto": "ETH",
            "reference": None
        },
        "message": "redeem_msg",
        "amount": None,
        "account_name": None,
        "card_name": None,
        "business_uuid": None,
        "descriptor": None,
        "processing_type": None,
        "source_id": None,
        "destination_id": None,
    }},
    {"no_content_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "version": "0.2",
            "crypto": "ETH",
            "reference": None
        }
    }},
    {"update_wallet_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "version": "0.2",
            "crypto": "ETH",
            "reference": None
        },
        "nickname": None,
        "default": None
    }},
    {"register_wallet_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "version": "0.2",
            "crypto": "ETH",
            "reference": None
        },
        "wallet_verification_signature": None,
        "wallet": {
            "blockchain_address": None,
            "blockchain_network": "ETH",
            "nickname": None,
            "default": None
        }
    }},
    {"get_wallets_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "version": "0.2",
            "crypto": "ETH",
            "reference": None
        },
        "search_filters": {
            "page": None,
            "per_page": None,
            "sort_ascending": None,
            "blockchain_network": None,
            "blockchain_address": None,
            "nickname": None
        }
    }},
    {"get_accounts_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "version": "0.2",
            "crypto": "ETH",
            "reference": None
        },
        "message": "get_accounts_msg"
    }},
    {"sila_balance_msg": {
        "address": None
    }},
    {"business_types_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "reference": None
        }
    }},
    {"business_roles_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "reference": None
        }
    }},
    {"naics_categories_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "reference": None
        }
    }},
    {"entity_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "version": "0.2",
            "crypto": "ETH",
            "reference": None
        },
        "message": "entity_msg",
        "address": {
            "address_alias": None,
            "street_address_1": None,
            "city": None,
            "state": None,
            "country": None,
            "postal_code": None,
            "street_address_2": None
        },
        "identity": {
            "identity_alias": None,
            "identity_value": None
        },
        "contact": {
            "contact_alias": None,
            "phone": None,
            "email": None,
            "sms_opt_in": None
        },
        "crypto_entry": {
            "crypto_alias": None,
            "crypto_code": "ETH",
            "crypto_address": None
        },
        "entity": {
            "first_name": None,
            "last_name": None,
            "entity_name": None,
            "birthdate": None,
            "relationship": None,
            "type": None,
            "business_type": None,
            "business_type_uuid": None,
            "business_website": None,
            "doing_business_as": None,
            "naics_code": None
        },
        "device": {
            "device_fingerprint": None,
            "session_identifier": None
        }
    }},
    {"link_business_member_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "business_handle": None,
            "reference": None
        },
        "role": None,
        "details": None,
        "ownership_stake": None
    }},
    {"unlink_business_member_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "business_handle": None,
            "reference": None
        },
        "role": None
    }},
    {"get_entity_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "reference": None
        }
    }},
    {"certify_beneficial_owner_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "business_handle": None,
            "reference": None
        },
        "member_handle": None,
        "certification_token": None
    }},
    {"certify_business_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "business_handle": None,
            "reference": None
        }
    }},
    {'get_transaction_msg': {
        "header": {
            "created": '',
            "app_handle": None,
            "user_handle": None,
            "version": None,
            "crypto": None,
            "reference": None
        },
        "message": "get_transactions_msg",
        "search_filters": {
            "transaction_id": None,
            "reference_id": None,
            "show_timelines": '',
            "sort_ascending": '',
            "max_sila_amount": '',
            "min_sila_amount": '',
            "statuses": [],
            "start_epoch": '',
            "end_epoch": '',
            "page": '',
            "per_page": '',
            "transaction_types": [],
            "bank_account_name":None,
            "processing_type": None,
        }
    }},
    {'cancel_transaction_msg': {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "version": None,
            "crypto": None,
            "reference": None
        },
        "transaction_id": None
    }},
    {"add_registration_data_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "reference": None
        },
        "email": None,
        "phone": None,
        "identity_alias": None,
        "identity_value": None,
        "address_alias": None,
        "street_address_1": None,
        "street_address_2": None,
        "city": None,
        "state": None,
        "postal_code": None,
        "country": None,
        "sms_opt_in": None,
        "device_fingerprint": None,
        "session_identifier": None
    }},
    {"update_registration_data_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "reference": None
        },
        "uuid": None,
        "email": None,
        "phone": None,
        "identity_alias": None,
        "identity_value": None,
        "address_alias": None,
        "street_address_1": None,
        "street_address_2": None,
        "city": None,
        "state": None,
        "postal_code": None,
        "country": None,
        "first_name": None,
        "last_name": None,
        "entity_name": None,
        "birthdate": None,
        "business_type": None,
        "naics_code": None,
        "doing_business_as": None,
        "business_website": None,
        "sms_opt_in": None
    }},
    {"delete_registration_data_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "reference": None
        },
        "uuid": None
    }},
    {"documents_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "version": "0.2",
            "crypto": "ETH",
            "reference": None
        },
        "message": None,
        "name": None,
        "filename": None,
        "hash": None,
        "mime_type": None,
        "document_type": None,
        "identity_type": None,
        "description": None
    }},
    {"list_documents_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "version": None,
            "crypto": None,
            "reference": None
        },
        "message": None,
        "start_date": None,
        "end_date": None,
        "doc_types": None,
        "search": None,
        "sort_by": None
    }},
    {"get_document_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "version": None,
            "crypto": None,
            "reference": None
        },
        "document_id": None
    }},
    {"document_types_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "version": None,
            "crypto": None,
            "reference": None
        }
    }},
    {"plaid_link_token_msg": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "reference": None
        },
        "android_package_name": None
    }},
    {"delete_account": {
        "header": {
            "created": None,
            "app_handle": None,
            "user_handle": None,
            "version": "0.2",
            "crypto": "ETH",
            "reference": None
        },
        "account_name": None
    }},
    {
        "check_partner_kyc": {
            "header": {
                "created": None,
                "app_handle": None,
                "version": "0.2",
                "crypto": "ETH",
                "reference": None
            },
            "query_app_handle": None,
            "query_user_handle": None
        }
    },
    {
        "update_account": {
            "header": {
                "created": None,
                "app_handle": None,
                "user_handle": None,
                "version": "0.2",
                "crypto": "ETH",
                "reference": None
            },
            "account_name": None,
            "new_account_name": None,
            "active": None
        }
    },
    {
        "plaid_update_link_token": {
            "header": {
                "created": None,
                "app_handle": None,
                "user_handle": None
            },
            "account_name": None,
        }
    },
    {
        'check_instant_ach': {
            "header": {
                "created": '',
                "app_handle": None,
                "user_handle": None,
                "version": "0.2",
                "crypto": "ETH",
                "reference": None
            },
            "account_name": None,
            "kyc_level": None
        }
    },
    {
        "get_institutions": {
            "header": {
                "created": None,
                "app_handle": None,
                "reference": None
            },
            "message": "header_msg",
            "search_filters": {
                "institution_name": None,
                "routing_number": None,
                "page": None,
                "per_page": None
            }
        }
    },
    {
        "link_card_msg": {
            "header": {
                "created": None,
                "app_handle": None,
                "user_handle": None,
                "version": "0.2",
                "crypto": "ETH",
                "reference": None
            },
            "message": "header_msg",
            "card_name": None,
            "account_postal_code": None,
            "token": None
        }
    },
    {
        "get_cards": {
            "header": {
                "created": None,
                "app_handle": None,
                "user_handle": None,
                "reference": None
            },
            "message": "header_msg"
        }
    },
    {
        "delete_card": {
            "header": {
                "created": None,
                "app_handle": None,
                "user_handle": None,
                "reference": None
            },
            "message": "header_msg",
            "card_name": None
        }
    },
    {
        "reverse_transaction_msg": {
            "header": {
                "created": None,
                "app_handle": None,
                "user_handle": None,
                "version": None,
                "crypto": None,
                "reference": None
            },
            "transaction_id": None
        }
    },
    {
        "get_webhooks": {
            "header": {
                "created": None,
                "app_handle": None,
                "user_handle": None,
                "reference": None,
                "version": "0.2",
                "crypto": "ETH"
            },
            "message": "header_msg",
            "search_filters": {
                "uuid": None,
                "delivered": None,
                "sort_ascending": None,
                "event_type": None,
                "endpoint_name": None,
                "user_handle": None,
                "start_epoch": None,
                "end_epoch": None,
                "page": None,
                "per_page": None
            }
        }
    },
    {
        "retry_webhook": {
            "header": {
                "created": None,
                "app_handle": None,
                "user_handle": None,
                "reference": None,
                "version": "0.2",
                "crypto": "ETH"
            },
            "message": "header_msg",
            "event_uuid": None,
        }
    },
    {
        "open_virtual_acc": {
            "header": {
                "created": None, 
                "auth_handle": None, 
                "app_handle": None,
                "user_handle": None, 
                "version": "0.2", 
                "crypto": "ETH", 
                "reference": None
            },
            "virtual_account_name": None
        }
    },
    {
        "get_virtual_acc": {
            "header": {
                "created": None, 
                "auth_handle": None, 
                "user_handle": None,
                "app_handle": None,
                "version": "0.2", 
                "crypto": "ETH", 
                "reference": None
            },
            "virtual_account_id": None
        }
    },
    {
        "get_virtual_accounts_msg": {
            "header": {
                "created": None, 
                "auth_handle": None, 
                "user_handle": None,
                "app_handle": None,
                "version": "0.2", 
                "crypto": "ETH", 
                "reference": None
            }
        }
    },
    {
        "update_virtual_account_msg": {
            "header": {
                "created": None, 
                "auth_handle": None, 
                "user_handle": None, 
                "app_handle": None,
                "version": "0.2", 
                "crypto": "ETH", 
                "reference": None
            },
            "virtual_account_id": None,
            "virtual_account_name": None,
            "active": None
        }
    },
    {
        "get_payment_methods":{
            "header": {
                "created": None,
                "auth_handle": None,
                "user_handle": None,
                "app_handle": None,
                "reference": None,
            }
        }
    }
]



from enum import Enum


class ProcessingTypes(str, Enum):
    STANDARD_ACH = 'STANDARD_ACH',
    SAME_DAY_ACH = 'SAME_DAY_ACH',
    INSTANT_ACH = 'INSTANT_ACH',
    CARD = "CARD",
    INSTANT_SETTLEMENT = "INSTANT_SETTLEMENT"



from typing import Optional


class UrlParameters():
    @staticmethod
    def add_query_parameter(parameter_name: str, parameter_value: str, query_parameters: Optional[str] = None) -> str:
        """
        Args:
            parameter_name (str): The name of the parameter to add in the query parameters
            parameter_value (str): The value of the parameter to add in the query parameters
            query_parameters (str): The current query parameters text
                (default is None)
        Returns:
            str: The query parameters with the new parameter added at the end of the text
        """
        if query_parameters is None or len(query_parameters.strip()) == 0:
            query_parameters = '?'
        else:
            query_parameters += '&'
        query_parameters += '{parameter_name}={parameter_value}'.format(
            parameter_name=parameter_name, parameter_value=parameter_value)
        return query_parameters



from .endpoints import endPoints
from silasdk import message
from silasdk.client import App
from silasdk.utils.url_parameters import UrlParameters
from typing import Optional
import warnings
import json
import requests


class User():
    def checkHandle(self, payload):
        """Check if the user handle is available.
        These endpoint returns the validity of a user handle
        Args:
        payload : Required user_handle to check if its available
        Returns:
        dict: response body (a confirmation message)
        """
        path = endPoints["checkHandle"]
        msg_type = "header_msg"
        response = message.postRequest(self, path, msg_type, payload)
        return response

    def register(self, payload):
        """Register a new user.
           This user will be kyced and ethereum address will be registered with sila 
        Args:
            payload : info about user like name,ssn, dob,ethereum address, ethereum handle etc
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["register"]
        msg_type = "entity_msg"
        response = message.postRequest(self, path, msg_type, payload)
        return response

    def requestKyc(self, payload, user_private_key, use_kyc_level=False):
        """Request kyc for a user by handle
           This user will be kyced and ethereum address will be registered with sila 
        Args:
            payload : info about user like name,ssn, dob,ethereum address, ethereum handle etc
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["requestKyc"]
        msg_type = "header_msg"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def linkAccount(self, payload, user_private_key, plaid=False):
        """link the bank account of user using plad
           This users bank account will be linked  
        Args:
            payload : need user handle and plad token
            user_private_key: users ethereum private key 
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["linkAccount"]
        msg_type = ("link_account_msg" if (plaid is False)
                    else "link_account_msg_plaid")
        if payload.get('public_token') is not None:
            warnings.warn(
                'public_token is deprecated in favor of plaid_token', DeprecationWarning)
            payload.update({
                "plaid_token": payload.get('public_token')
            })
            payload.pop('public_token')
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def checkKyc(self, payload, user_private_key):
        """check if the user has been kyced.
            The user will be checked if the they have been kyced
        Args:
            payload : includes 
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["checkKyc"]
        msg_type = "header_msg"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def addIdentity(self, payload, user_private_key):
        """change the info about user like change ssn, email ,etc.
            The used will be checked if the they have been kyced
        Args:
            payload : includes information to be edited and usee handle
            user_private_key: users ethereum private key 
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["addIdentity"]
        msg_type = "identity_msg"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def getAccounts(self, payload, user_private_key):
        """get the accounts of users registered with sila
            The user will be checked if they have been kyced, along with app
        Args:
            user_hanlde: users handle registered with app
            user_private_key: user private key asscoicated with crypto address
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["getAccounts"]
        msg_type = "get_accounts_msg"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def getAccountBalance(self, payload, user_private_key):
        """get the account balance of a user registered with sila
        Args:
            user_hanlde: users handle registered with app
            user_private_key: user private key asscoicated with crypto address
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["getAccountBalance"]
        msg_type = "account_name_msg"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def openVirtualAccount(self, payload, user_private_key):
        """get the accounts of users registered with sila
            The user will be checked if they have been kyced, along with app
        Args:
            user_hanlde: users handle registered with app
            user_private_key: user private key asscoicated with crypto address
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["openVirtualAccount"]
        msg_type = "open_virtual_acc"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response
        
    def getVirtualAccount(self, payload, user_private_key):
        """get the accounts of users registered with sila
            The user will be checked if they have been kyced, along with app
        Args:
            user_hanlde: users handle registered with app
            user_private_key: user private key asscoicated with crypto address
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["getVirtualAccount"]
        msg_type = "get_virtual_acc"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def getVirtualAccounts(self, payload, user_private_key):
        """get the accounts of users registered with sila
            The user will be checked if they have been kyced, along with app
        Args:
            user_hanlde: users handle registered with app
            user_private_key: user private key asscoicated with crypto address
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["getVirtualAccounts"]
        msg_type = "get_virtual_accounts_msg"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def updateVirtualAccount(self, payload, user_private_key):
        """get the accounts of users registered with sila
            The user will be checked if they have been kyced, along with app
        Args:
            user_hanlde: users handle registered with app
            user_private_key: user private key asscoicated with crypto address
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["updateVirtualAccount"]
        msg_type = "update_virtual_account_msg"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def getPaymentMethods(self, payload, user_private_key):
        """get the accounts of users registered with sila
            The user will be checked if they have been kyced, along with app
        Args:
            user_hanlde: users handle registered with app
            user_private_key: user private key asscoicated with crypto address
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["getPaymentMethods"]
        msg_type = "get_payment_methods"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    @staticmethod
    def getTransactions(app: App, payload: dict, user_private_key: str) -> dict:
        """get the users transactions registered with ur app
           The user will be checked if they have been kyced, along with app
        Args:
            user_hanlde: users handle registered with app
            user_private_key: user private key asscoicated with crypto address
        Returns:
            dict: response body (a confirmation message)
        """
        warnings.warn(
            'This method has been deprecated in favor of get_transactions', DeprecationWarning)
        return User.get_transactions(app, payload, user_private_key)

    @staticmethod
    def get_transactions(app: App, payload: dict, user_private_key: str = '') -> dict:
        if user_private_key != '':
            warnings.warn(
                'user_private_key is no longer needed, this will be removed in future versions.', DeprecationWarning)
        path = endPoints["getTransactions"]
        msg_type = "get_transaction_msg"
        response = message.postRequest(
            app, path, msg_type, payload)
        return response

    def silaBalance(self, address):
        """get sila balance of the addresses registered with sila
           The user will be checked if they have been kyced, along with app
        Args:
            address: requires valid ethereum address
        Returns:
            dict: response body (a confirmation message)
        """
        data = json.dumps({"address": str(address)})
        header = {'content-type': 'application/json'}
        if self.tier == "prod":
            endpoint = endPoints["silaBalanceProd"]
        elif self.tier == "sandbox":
            endpoint = endPoints["silaBalanceSandbox"]
        response = requests.post(endpoint, data=data, headers=header)
        return response.json()

    def getSilaBalance(self, address):
        """get sila balance of the addresses registered with sila
           The user will be checked if they have been kyced, along with app
        Args:
            address: requires valid ethereum address
        Returns:
            dict: response body (a confirmation message)
        """
        payload = {"address": str(address)}
        path = endPoints["getSilaBalance"]
        msg_type = "sila_balance_msg"
        response = message.postRequest(
            self, path, msg_type, payload)
        return response

    def getEntities(self, payload, per_page=None, page=None):
        """Return all end-user and legal entities (businesses) associated with a customer application.
            This endpoint allows the listing of all entities registered to an application.
        Args:
            payload: filters information
            per_page: pagination information
            page: pagination information
        Returns:
            dict: response body (entities list)
        """
        path = endPoints["getEntities"] + (('&per_page=' + str(per_page)) if per_page is not None else '') + (
            ('&page=' + str(page)) if page is not None else '')
        msg_type = "header_msg"
        response = message.postRequest(
            self, path, msg_type, payload)
        return response

    @staticmethod
    def getEntity(app: App, payload: dict, user_private_key: str, pretty_dates: Optional[bool] = None) -> dict:
        """
        Args:
            app (App): The current app configuration
            payload (dict): filters information
            user_private_key (str): The user's private key to sign the message
            pretty_dates (bool): Indicates if you want pretty dates in the response object
        Returns:
            dict: response body (entity information)
        """
        warnings.warn(
            'This method is deprecated in favor of get_entity', DeprecationWarning)
        return User.get_entity(app, payload, user_private_key)

    @staticmethod
    def get_entity(app: App, payload: dict, user_private_key: str, pretty_dates: Optional[bool] = None) -> dict:
        """
        Args:
            app (App): The current app configuration
            payload (dict): Filters information
            user_private_key (str): The user's private key to sign the message
            pretty_dates (bool): Indicates if you want pretty dates in the response object
        Returns:
            dict: response body (entity information)
        """
        path = endPoints["getEntity"]
        if pretty_dates:
            path += UrlParameters.add_query_parameter(
                "pretty_dates", "true")
        msg_type = "get_entity_msg"
        response = message.postRequest(
            app, path, msg_type, payload, user_private_key)
        return response

    @staticmethod
    def addRegistrationData(app: App, registration_field: str, payload: dict, user_private_key: str) -> dict:
        """
        Args:
            payload: registration data
            user_private_key
        Returns:
            dict: response body (entity information)
        """
        warnings.warn(
            'This method has been deprecated in favor of add_registration_data', DeprecationWarning)
        return User.add_registration_data(app, registration_field, payload, user_private_key)

    @staticmethod
    def add_registration_data(app: App, registration_field: str, payload: dict, user_private_key: str) -> dict:
        path = endPoints["addRegistrationData"] + registration_field
        msg_type = "add_registration_data_msg"
        response = message.postRequest(
            app, path, msg_type, payload, user_private_key)
        return response

    @staticmethod
    def updateRegistrationData(app: App, registration_field: str, payload: dict, user_private_key: str):
        """
        Args:
            app (App): The application configuration
            registration_field (str): The specific registration field to update
            payload (dict): registration data
            user_private_key (str): The user's private key to sign the message
        Returns:
            dict: response body (entity information)
        """
        warnings.warn(
            'This method has been deprecated in favor of update_registration_data', DeprecationWarning)
        return User.update_registration_data(app, registration_field, payload, user_private_key)

    @staticmethod
    def update_registration_data(app: App, registration_field: str, payload: dict, user_private_key: str) -> dict:
        path = endPoints["updateRegistrationData"] + registration_field
        msg_type = "update_registration_data_msg"
        response = message.postRequest(
            app, path, msg_type, payload, user_private_key)
        return response

    def deleteRegistrationData(self, registration_field, payload, user_private_key):
        """
        Args:
            payload: registration data
            user_private_key
        Returns:
            dict: response body (entity information)
        """
        path = endPoints["deleteRegistrationData"] + registration_field
        msg_type = "delete_registration_data_msg"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    @staticmethod
    def plaid_link_token(app: App, user_handle: str, android_package_name: str = None) -> dict:
        path = endPoints["plaid_link_token"]
        msg_type = "plaid_link_token_msg"
        payload = {"user_handle": user_handle}
        if android_package_name is not None:
            payload["android_package_name"] = android_package_name
        response = message.postRequest(
            app, path, msg_type, payload)
        return response

    @staticmethod
    def delete_account(app: App, payload: dict, user_private_key: str) -> dict:
        path = endPoints["delete_account"]
        msg_type = "delete_account"
        response = message.postRequest(
            app, path, msg_type, payload, user_private_key)
        return response

    @staticmethod
    def check_partner_kyc(app: App, payload: dict) -> dict:
        path = "/check_partner_kyc"
        msg_type = "check_partner_kyc"
        response = message.postRequest(
            app, path, msg_type, payload)
        return response

    @staticmethod
    def update_account(app: App, payload: dict, user_private_key: str) -> dict:
        path = '/update_account'
        msg_type = "update_account"
        response = message.postRequest(
            app, path, msg_type, payload, user_private_key)
        return response

    @staticmethod
    def plaid_update_link_token(app: App, payload: dict) -> dict:
        path = endPoints["plaid_update_link_token"]
        msg_type = "plaid_update_link_token"
        response = message.postRequest(
            app, path, msg_type, payload)
        return response

    @staticmethod
    def check_instant_ach(app: App, payload: dict, user_private_key: str) -> dict:
        path = '/check_instant_ach'
        msg_type = "check_instant_ach"
        response = message.postRequest(
            app, path, msg_type, payload, user_private_key)
        return response

    @staticmethod
    def get_institutions(app: App, payload: dict) -> dict:
        path = '/get_institutions'
        msg_type = "get_institutions"
        response = message.postRequest(
            app, path, msg_type, payload)
        return response

    @staticmethod
    def link_card(app: App, payload: dict, user_private_key: str) -> dict:
        path = '/link_card'
        msg_type = "link_card_msg"
        response = message.postRequest(
            app, path, msg_type, payload, user_private_key)
        return response

    @staticmethod
    def get_cards(app: App, payload: dict, user_private_key: str) -> dict:
        path = '/get_cards'
        msg_type = "get_cards"
        response = message.postRequest(
            app, path, msg_type, payload, user_private_key)
        return response

    @staticmethod
    def delete_card(app: App, payload: dict, user_private_key: str) -> dict:
        path = '/delete_card'
        msg_type = "delete_card"
        response = message.postRequest(
            app, path, msg_type, payload, user_private_key)
        return response

    @staticmethod
    def get_webhooks(app: App, payload: dict, user_private_key: str) -> dict:
        path = '/get_webhooks'
        msg_type = "get_webhooks"
        response = message.postRequest(
            app, path, msg_type, payload, user_private_key)
        return response

    @staticmethod
    def retry_webhook(app: App, payload: dict, user_private_key: str) -> dict:
        path = '/retry_webhook'
        msg_type = "retry_webhook"
        response = message.postRequest(
            app, path, msg_type, payload, user_private_key)
        return response


import warnings

from silasdk.endpoints import endPoints
from silasdk.client import App
from silasdk.message import postRequest


class Transaction():

    @staticmethod
    def issueSila(app: App, payload: dict, user_private_key: str) -> dict:
        """issues sila erc20token for dollar amount on ethereum blockchain to kyced ethereum addresses (price one cent per token)
            the handle address signatures need to be verified
        Args:
            payload : includes user handle and amount
            user_private_key: users ethereum private key 
        Returns:
            dict: response body (a confirmation message)
        """
        warnings.warn(
            'This method is deprecated in favor of issue_sila', DeprecationWarning)
        return Transaction.issue_sila(app, payload, user_private_key)

    @staticmethod
    def issue_sila(app: App, payload: dict, user_private_key: str) -> dict:
        path = endPoints["issueSila"]
        msg_type = "issue_msg"
        response = postRequest(
            app, path, msg_type, payload, user_private_key)
        return response

    def redeemSila(self, payload, user_private_key):
        """redeems sila erc20token for dollar amount on ethereum blockchain to kyced ethereum addresses (price one cent per token)
            the handle address signatures need to be verified
        Args:
            payload : user handle and amount
            user_private_key: users ethereum private key 
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["redeemSila"]
        msg_type = "redeem_msg"
        response = postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def transferSila(self, payload, user_private_key, use_destination_address=False):
        """ transfer sila from one ethereum address to another using sila api
            the handle address signatures need to be verified
        Args:
            payload : user handle and amount
            user_private_key: users ethereum private key 
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["transferSila"]
        msg_type = "transfer_msg"
        response = postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def plaidSamedayAuth(self, payload, user_private_key):
        """ Handle a request for a Plaid public_token in order to complete
            Plaid's Same Day Microdeposit Authentication
        Args:
            payload : user handle and amount
            user_private_key: users ethereum private key
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["plaidSameDayAuth"]
        msg_type = "account_name_msg"
        response = postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def cancelTransaction(self, payload, user_private_key):
        """ Cancel a pending transaction under certain circumstances


        Args:
            payload : user handle and transaction id
            user_private_key: users ethereum private key
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["cancelTransaction"]
        msg_type = "cancel_transaction_msg"
        response = postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def reverseTransaction(self, payload, user_private_key):
        """ Reverse transaction under certain circumstances
        Args:
            payload : user handle and transaction id
            user_private_key: users ethereum private key
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["reverseTransaction"]
        msg_type = "reverse_transaction_msg"
        response = postRequest(
            self, path, msg_type, payload, user_private_key)
        return response



from enum import Enum


class RegistrationFields(str, Enum):
    EMAIL = 'email',
    PHONE = 'phone',
    IDENTITY = 'identity',
    ADDRESS = 'address',
    ENTITY = 'entity',
    DEVICE = 'device'
